/************************************************************************/
/*                                                                      */
/*                      Debouncing 8 Keys								*/
/*			Sampling 4 Times											*/
/*			With Repeat Function										*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/* Forum http://www.mikrocontroller.net/topic/48465#new                 */
/*                                                                      */
/* Erlaubte Funktionen einer Taste:                                     */
/*																		*/
/* - get_key_press()                                                    */
/*                                                                      */
/* - get_key_rpt()                                                      */
/*                                                                      */
/* - get_key_press() mit get_key_rpt()                                  */
/*                                                                      */
/* - get_key_short() mit get_key_long()                                 */
/*                                                                      */
/* - get_key_short() mit get_key_long_r() und get_key_rpt_l()           */
/************************************************************************/
/*                                                                    
 Modus 1 ('get_key_press' mit 'get_key_rpt')                        
 ===========================================                        


Zeitachse ---->
                 __      _________________      __
keystate    ____/  \____/                 \____/  \____
key_press   ----X-------X----------------------X-------
key_rpt     --------------------X--X--X--X-------------
key_release -------X----------------------X-------X----
                        |       |  |  |
                        |       |__|__|
                        |       | \ /
                        |_______|  REPEAT_NEXT
                              \
                               REPEAT_START



Modus 2 ('get_key_short' mit 'get_key_long')
============================================

                 __      _________________      __
keystate    ____/  \____/                 \____/  \____
key_short   -------X------------------------------X----
key_long    --------------------X----------------------
key_release -------X----------------------X-------X----



Modus 3 ('get_key_short' mit 'get_key_long_r' und 'get_key_rpt_l')
==================================================================

                 __      _________________      __
keystate    ____/  \____/                 \____/  \____
key_short   -------X------------------------------X----
key_long_r  --------------------X----------------------
key_rpt_l   -----------------------X--X--X-------------
key_release -------X----------------------X-------X----

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include "inc/globals.h"  // before delay.h, da F_CPU benoetigt wird
#include <util/delay.h>   // for delay functions, requires F_CPU definition
#include "inc/keys.h"



volatile uint8_t key_state;				// debounced and inverted key state:
					                // bit = 1: key pressed
volatile uint8_t key_press;				// key press detect; volatile, da durch ISR geaendert

volatile uint8_t key_release;            // key release detect

volatile uint8_t key_rpt;				// key long press and repeat


ISR( TIM0_COMPA_vect )			// every 10ms
{
    static uint8_t ct0 = 0xFF, ct1 = 0xFF, rpt;
    uint8_t i;

    i = key_state ^ ~KEY_PIN;                     // key changed ?
    ct0 = ~( ct0 & i );                           // reset or count ct0
    ct1 = ct0 ^ (ct1 & i);                        // reset or count ct1
    i &= ct0 & ct1;                               // count until roll over ?
    key_state ^= i;                               // then toggle debounced state
    key_press |= key_state & i;                   // 0->1: key press detect
	key_release |= ~key_state & i;                // 1->0: key release detect

    if( (key_state & REPEAT_MASK) == 0 )          // check repeat function
    rpt = REPEAT_START;                           // start delay
    if( --rpt == 0 ){
	    rpt = REPEAT_NEXT;                        // repeat delay
	    key_rpt |= key_state & REPEAT_MASK;
    }
}


uint8_t get_key_press( uint8_t key_mask )
{
    ATOMIC_BLOCK(ATOMIC_FORCEON){
	    key_mask &= key_press;                      // read key(s)
	    key_press ^= key_mask;                      // clear key(s)
    }
    return key_mask;
}

uint8_t get_key_release( uint8_t key_mask )
{
	ATOMIC_BLOCK(ATOMIC_FORCEON){
	  key_mask &= key_release;   // read key(s)
	  key_release ^= key_mask;   // clear key(s)
	}
	return key_mask;
}


uint8_t get_key_rpt( uint8_t key_mask )
{
    ATOMIC_BLOCK(ATOMIC_FORCEON){
	    key_mask &= key_rpt;                        // read key(s)
	    key_rpt ^= key_mask;                        // clear key(s)
    }
    return key_mask;
}


uint8_t get_key_short( uint8_t key_mask )
{
    uint8_t i;

    ATOMIC_BLOCK(ATOMIC_FORCEON)
    i = get_key_press( ~key_state & key_mask );
    return i;
}


uint8_t get_key_long( uint8_t key_mask )
{
  return get_key_press( get_key_rpt( key_mask ));
}


uint8_t get_key_long_r( uint8_t key_mask )      // if repeat function needed
{
	return get_key_press( get_key_rpt( key_press & key_mask ));
}


uint8_t get_key_rpt_l( uint8_t key_mask )       // if long function needed
{
	return get_key_rpt( ~key_press & key_mask );
}

/*
int main( void )
{
   TCCR0A = 1<<WGM01;                                    // Mode 2. CTC
   TCCR0B = 1<<CS02 | 1<<CS00;                           // F_CPU / 1024
   OCR0A = F_CPU / 1024.0 * 10e-3 - 0.5;                 // 10ms interrupt
   TIMSK0 = 1<<OCIE0A;									 // TIMSK0 for ATTINY13, else TIMSK for ATTINY 25/45/85

   LED_PORT = 1<<LED0 | 1<<LED1 | 1<<LED2 | 1<<LED3;     // LEDs off
   LED_DDR = 1<<LED0 | 1<<LED1 | 1<<LED2 | 1<<LED3;
   KEY_PORT |= 1<<KEY0;                                  // Pullup on
   sei();

   for(;;){                                              // main loop

	   if( get_key_short( 1<<KEY0 ))
	   LED_PORT ^= 1<<LED0;

	   if( get_key_long_r( 1<<KEY0 ))
	   LED_PORT ^= 1<<LED1;

	   if( get_key_rpt_l( 1<<KEY0 ))
	   LED_PORT ^= 1<<LED2;
   }
}
*/
