//----------------------------------------------------------------------
// Titel     : DSP Heimradio mit ATTINY13A/85 fuer SI4735 Radio
//----------------------------------------------------------------------
// Funktion  : SI4735 control functions
// Includes  : i2cmaster.h, si4735.h 
// Hinweis   : i2cmaster.S (Assem. Source) den source Files hinzuf.
//----------------------------------------------------------------------
// Sprache   : C
// Datum     : 11.04.2014 23:59:33
// Version   : 1.01
// Autor     : Andreas Nestler
// Changes   : 
//		v1   : initial version; Si4735 extracted control functions
//      v1.01: some small updates to avoid globals and extern
//      v1.03: header file update after header files cleanup
//----------------------------------------------------------------------

#include "inc/globals.h"  // before delay.h, da F_CPU benoetigt wird
#include <util/delay.h>   // for delay functions, requires F_CPU definition
#include "inc/si4735.h"
#include "inc/i2cmaster.h" // for i2c functions

/* initialize the Si4735 */
void si4735init(void)
{
	i2c_start_wait(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x01);                 // Device power-up
    i2c_write(0x10);                 // FUNC, XOSCEN -> FM, 32kHz Oscillator on
	i2c_write(0x05);                 // OPMODE: Analog audio outputs
    i2c_stop();
	_delay_ms(100);					 // Pause von 100ms ntig, sonst ist GPO CTL setting nicht mglich
	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
	i2c_write(0x80);                 // SET_PROPERTY
	i2c_write(0x02);                 // GPO1 as output
	i2c_stop();                 
    _delay_ms(500);                  // check for wait_for_ready_CTS replacement (?!)
}

/*---------------------------------------------------------------------------*/
/* Some property settings */
/*---------------------------------------------------------------------------*/

void fm_deemphasis(void)        //FM Deemphasis einstellen
{             
   i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
   i2c_write(0x12);
   i2c_write(0x00);     
   i2c_write(0x11);
   i2c_write(0x00);           
   i2c_write(0x00);    
   i2c_write(0x01);     //Deemphasis = 50s -> valid for european FM
   i2c_stop();      
   wait_for_ready_CTS();
   _delay_ms(1);
}
        
void fm_max_tune_error(void)       //FM Max Tune Error
{
   i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
   i2c_write(0x12);
   i2c_write(0x00);     
   i2c_write(0x11);
   i2c_write(0x08);           
   i2c_write(0x00);    
   i2c_write(0x14);     //max Tune Error 14=20kHz (default), 28=40KHz, 32=50KHz
   i2c_stop();      
   wait_for_ready_CTS();
   _delay_ms(1);
}

void fm_seek_tune_snr_threshold(void)        //FM_SEEK_TUNE_SNR_THRESHOLD
{      
   i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
   i2c_write(0x12);
   i2c_write(0x00);     
   i2c_write(0x14);
   i2c_write(0x03);           
   i2c_write(0x00);    
   i2c_write(0x06);     //FM_SEEK_TUNE_SNR_THRESHOLD 6dB (3dB def)
   i2c_stop();      
   wait_for_ready_CTS();
   _delay_ms(1);
}

void fm_seek_tune_rssi_threshold(void)       //FM_SEEK_TUNE_RSSI_THRESHOLD
{
   i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
   i2c_write(0x12);
   i2c_write(0x00);     
   i2c_write(0x14);
   i2c_write(0x04);           
   i2c_write(0x00);    
   i2c_write(0x14);     //FM_SEEK_TUNE_RSSI_THRESHOLD 20dBuV (def)
   i2c_stop();      
   wait_for_ready_CTS();
   _delay_ms(1);
}

void fm_seek_freq_spacing(void)        //FM_SEEK_FREQ_SPACING
{
	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
	i2c_write(0x12);
	i2c_write(0x00);
	i2c_write(0x14);
	i2c_write(0x02);
	i2c_write(0x00);
	i2c_write(0x05);     //FM_SEEK_FREQ_SPACING 50 KHz, default is 0x0A -> 100kHz
	i2c_stop();
	wait_for_ready_CTS();
	_delay_ms(1);
}

/* Set tuning frequency */
void si4735freq(unsigned int iF_)
{
	unsigned char cLow;
	unsigned char cHigh;

	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x20);                 // write value
    i2c_write(0x00);                 // write value
    cHigh = iF_ >> 8;			 	 // High Byte of iFreq
    cLow =  iF_ & 0xFF;			 	 // Low Byte of iFreq
	i2c_write(cHigh);
	i2c_write(cLow);
	i2c_write(0x00);                 // write value
    i2c_stop();
	wait_for_ready_CTS();			 // wait for clear to send
//    _delay_ms(500);  
}


void Scan(void)
{
	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
    i2c_write(0x21);                 // FM_SEEK_START
    i2c_write(0x08);                 // SEEK UP , for Wrap function write 0x0C
    i2c_stop();
//    _delay_ms(500);  
	_delay_ms(60);
	wait_for_ready_CTS();
}

/* Read Frequency from Fm_Tune_Status function */
unsigned int Fm_tune_status(void)
{
  unsigned int iF_;
  unsigned char R1, R2, R3;

  UNUSED(R1);						// avoid compiler warning about unused var
  i2c_start(Dev4735+I2C_WRITE);		// set device address and write mode
  i2c_write(0x22);                 	// command Fm_Tune_status
  i2c_write(0x03);					// command argument 1
  i2c_stop();
  i2c_start(Dev4735+I2C_READ);    	// set device address and read mode
  R1=i2c_readAck();					// dummy read => STATUS
  R1=i2c_readAck();					// Response 1
  R2=i2c_readAck();					// Response 2 -> Freq H
  R3=i2c_readAck();					// Response 3 -> Freq L
  R1=i2c_readNak();					// Response 4 -> RSSI
									// Response 5 - 7 not read
  iF_ = R2 << 8;				 	// High Byte creation
  iF_ += R3;						// Low byte add
  i2c_stop();
  return iF_;
}

/* Read out receive signal quality */
tRSQ Fm_rsq_status(void)
{
  tRSQ rsq_;
  unsigned char R1;

  UNUSED(R1);						// avoid compiler warning about unused var
  i2c_start(Dev4735+I2C_WRITE);		// set device address and write mode
  i2c_write(0x23);                 	// write value for fm_rsq_status
  i2c_write(0x01);					// INT status cleared
  i2c_stop();
  wait_for_ready_CTS();				// wait for clear to send -> needed!
  i2c_start(Dev4735+I2C_READ);    	// set device address and read mode
  R1=i2c_readAck();					// dummy read => STATUS
  R1=i2c_readAck();					// Response 1
  R1=i2c_readAck();					// Response 2
  rsq_.stblend=i2c_readAck();		// Response 3 -> PILOT + STBLEND
  rsq_.rssi=i2c_readAck();			// Response 4 -> RSSI
  rsq_.snr=i2c_readAck();			// Response 5 -> SNR
  R1=i2c_readAck();					// Response 6
  R1=i2c_readNak();					// Response 7
  i2c_stop();
  return rsq_;
}

/* Mono control */
void Mono_Ctrl( unsigned char cBP_Flag)
{
  unsigned char cMono_Blend_Th;
  
  if (!cBP_Flag) {  // MONO as standard
	cMono_Blend_Th = 0x7F; // 127 -> MONO forced
  }
  else {
	cMono_Blend_Th = 0x1E; // 0 -> Stereo forced; 30uV -0x1E Threshold - default
  };

  /*Set into Mono mode */
  i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
  i2c_write(0x12);                 // SET_PROPERTY
  i2c_write(0x00);				   // ARG 1 always 0 for properties
  i2c_write(0x11);                 // High byte of 0x1106 FM_BLEND_MONO
  i2c_write(0x06);                 // Low byte of 0x1106 FM_BLEND_MONO
  i2c_write(0x00);                 // High byte of property value -> 0x00
  i2c_write(cMono_Blend_Th);       // Low byte of property value -> 7F / 1E
  i2c_stop(); 
  wait_for_ready_CTS();
}

/* Loudness control */
void Vol_Ctrl( unsigned char cVol)
{
  i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
  i2c_write(0x12);                 // SET_PROPERTY
  i2c_write(0x00);				   // ARG 1 always 0 for properties
  i2c_write(0x40);                 // High byte of 0x4000 Volume ctrl
  i2c_write(0x00);                 // Low byte of 0x4000 Volume ctrl
  i2c_write(0x00);                 // High byte of property value -> 0x00
  i2c_write(cVol);                 // Low byte of property value -> 3F / 3C
  i2c_stop();
  wait_for_ready_CTS();
}


void GPO_set(unsigned char cSetVal)
{
 	i2c_start(Dev4735+I2C_WRITE);    // set device address and write mode
	i2c_write(0x81);                 // SET_PROPERTY
	i2c_write(cSetVal);              // set output value of GPOs
	i2c_stop();     
}

/* Clear to send function */
void wait_for_ready_CTS(void)
{
	unsigned char cReady=0;  // local defined var saves memory, no need for global var
	
	while(!(cReady & 0x80))
	{
		i2c_start(Dev4735+I2C_READ);
		cReady=i2c_readNak();
		i2c_stop();
	}
	cReady=0;
}

